package Mqtt.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.*;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 本地文件缓存工具类，用来存储 MQTT 未能成功发送的消息，并在连接恢复后重新读取和发送。
 */
@Component
public class LocalCacheManager {

    private final Path cacheDir;

    /**
     * 构造函数，创建缓存目录。
     *指定缓存目录为 "mqtt-cache"（可在配置文件中外部化）。
     * 所有缓存的消息都以 JSON 文件的形式保存在这个目录
     * @param cachePath 缓存目录路径
     * @throws IOException 创建目录时发生 IO 异常
     */
    public LocalCacheManager(@Value("${cache.path:./cache}") String cachePath) throws IOException {
        this.cacheDir = Paths.get(cachePath);
        Files.createDirectories(cacheDir);
    }

    /**
     * 缓存一条消息。
     * @param json 要缓存的消息，以 JSON 格式表示
     */
    public void cache(String json) {
        try {
            String file = "msg-" + System.currentTimeMillis() + ".json";
            Files.write(cacheDir.resolve(file), json.getBytes(StandardCharsets.UTF_8));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 加载所有缓存的消息。
     * @return 所有缓存的消息，以 JSON 格式表示
     */
    public List<String> loadAll() {
        try (Stream<Path> files = Files.list(cacheDir)) {
            return files
                    .filter(p -> p.toString().endsWith(".json"))
                    .map(p -> {
                        try {
                            return new String(Files.readAllBytes(p), StandardCharsets.UTF_8);
                        } catch (IOException e) {
                            return null;
                        }
                    })
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList());
        } catch (IOException e) {
            e.printStackTrace();
            return Collections.emptyList();
        }
    }

    /**
     * 清空所有缓存的消息。
     */
    public void clearAll() {
        try (Stream<Path> files = Files.list(cacheDir)) {
            files.forEach(p -> p.toFile().delete());
        } catch (IOException ignored) {}
    }
}

