package Mqtt.service;

import Mqtt.config.LocalCacheManager;
import Mqtt.config.MQTTConfig;
import Mqtt.config.PushCallback;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.stereotype.Component;

/**
 * 订阅端逻辑
 */
@Component
@Slf4j
public class MqttSubscriberManager {

    private final MQTTConfig config;
    private final LocalCacheManager cacheManager;
    private MqttClient client;

    public MqttSubscriberManager(MQTTConfig config, LocalCacheManager cacheManager) {
        this.config = config;
        this.cacheManager = cacheManager;
        connect();
    }

    private void connect() {
        try {
            String topic = config.getSubscriber().getDefaultTopic();
            if (topic == null) {
                throw new IllegalArgumentException("订阅主题 defaultTopic 不能为空，请检查配置文件！");
            }


            client = new MqttClient(config.getHost(), config.getSubscriber().getClientId(), new MemoryPersistence());
            client.setCallback(new PushCallback(cacheManager));

            MqttConnectOptions options = new MqttConnectOptions();
            options.setCleanSession(config.isCleansession());
            options.setUserName(config.getUsername());
            options.setPassword(config.getPassword().toCharArray());
            options.setKeepAliveInterval(config.getKeepalive());
            options.setConnectionTimeout(config.getConnectionTimeout());

            client.connect(options);
            client.subscribe(config.getSubscriber().getDefaultTopic());

            log.info("订阅端连接成功并订阅：{}", config.getSubscriber().getDefaultTopic());

        } catch (MqttException e) {
            log.error("订阅端连接失败：{}", e.getMessage(), e);
        }
    }


    public void subscribe(String topic, int qos) {
        try {
            if (client != null && client.isConnected()) {
                client.subscribe(topic, qos);
                log.info("手动订阅成功：{} (QoS: {})", topic, qos);
            } else {
                log.warn("订阅端尚未连接，无法订阅：{}", topic);
            }
        } catch (MqttException e) {
            log.error("订阅主题失败：{}", topic, e);
        }
    }

}

